module(..., package.seeall)

require "core"
require "lpeg"
require "re"
utilities = require("utilities")
generate_SPARQL= require("generate_SPARQL")

local triplesSameSubjectTable = {}
local classTriplesTable = {}
local filterRelationalExpressionTable = {}
local notExistsFunc = false
local filterTrueCount = 0
local relationalExprCount = 0
local className
local attributeAlias
local classInProc
local isAggregate = false
local isFunction = false
local attributesNamesTable = {}
local attributesNamesTableP = {}
local prefixes = {}
local idTable = {}
local pathInString = false

function sparqlExpressionGrammar()
	local grammar = re.compile([[
		Main <- (Expression space !.)
		-- Expression <- ({:ConditionalOrExpression: ConditionalOrExpression :}) -> {}
		Expression <- ({:OrExpression: OrExpression :}) -> {}
		
		OrExpression <- (ANDExpression  ( space 'OR' space ANDExpression )*) -> {}
		ANDExpression <- ({:ANDExpression: ANDExpressionA :}) -> {}
		ANDExpressionA <- (ConditionalOrExpressionA (space 'AND' space ConditionalOrExpressionA )*) -> {}
		ConditionalOrExpressionA <- ({:ConditionalOrExpression: ConditionalOrExpression :}) -> {}
		ConditionalOrExpression <- 	(ConditionalAndExpression  ( space '||' space ConditionalAndExpression )*) -> {}
		ConditionalAndExpression <- ({:ConditionalAndExpression: ValueLogicalA :}) -> {}
		ValueLogicalA <- (ValueLogical (space '&&' space ValueLogical )*) -> {}
		ValueLogical <- ({:RelationalExpression: RelationalExpression :}) -> {}
		RelationalExpression <- ({:NumericExpressionL: NumericExpression:} ((space {:Relation: '=' :} space {:NumericExpressionR: NumericExpression:} )
				/ (space {:Relation: '!=' :} space {:NumericExpressionR: NumericExpression:} )
				/ (space {:Relation: '<' :} space {:NumericExpressionR: NumericExpression:} )
				/ (space {:Relation: '>'  :} space {:NumericExpressionR: NumericExpression:} )
				/ (space {:Relation: '<=' :} space {:NumericExpressionR: NumericExpression:} )
				/ (space {:Relation: '>=' :} space {:NumericExpressionR: NumericExpression:} )
				/ (space {:Relation: '<>' :} space {:NumericExpressionR: NumericExpression:} )
				/ (space {:Relation: 'IN' :} space  ExpressionList2 )
				/ (space {:Relation: 'NOT' space 'IN' :} space ExpressionList2 ))?) ->{}
		NumericExpression <- ( ({:AdditiveExpression: AdditiveExpression:})) -> {}
		
		ConcatExprassion <-(MultiplicativeExpression ("++" MultiplicativeExpression)*)->{}
		
		AdditiveExpression <- 	({:MultiplicativeExpression: MultiplicativeExpression:} ( {:MultiplicativeExpressionList: MultiplicativeExpressionListA:} )) -> {}
		MultiplicativeExpressionListA <- (( MultiplicativeExpressionList )*)->{}
		MultiplicativeExpressionList <- ((space {:Concat: '++' :} space {:MultiplicativeExpression: MultiplicativeExpression:}) 
									/ (space {:Additive: '+' :} space {:MultiplicativeExpression: MultiplicativeExpression:})
									/ (space {:Additive: '-' :} space {:MultiplicativeExpression: MultiplicativeExpression:})
									/ ( NumericLiteralPositive / NumericLiteralNegative ) 
									( (space '*' space UnaryExpression ) / (space '/' space UnaryExpression ) )*) -> {}
		MultiplicativeExpression <- ({:UnaryExpression: UnaryExpression:} ( {:UnaryExpressionList: UnaryExpressionListA:})) -> {}
		UnaryExpressionListA <-(UnaryExpressionList*) -> {}
 		UnaryExpressionList <- ((space {:Unary: '*' :}space  {:UnaryExpression: UnaryExpression:} )
							/ (space {:Unary: '/' :} space {:UnaryExpression: UnaryExpression:}) ) -> {}
		UnaryExpression <- ((space {:Additive: '!' :} space {:PrimaryExpression: PrimaryExpression:}) 
							--/(space {:Additive: '+' :} space {:PrimaryExpression: PrimaryExpression:})
							/(space {:Additive: '-' :} space {:PrimaryExpression: PrimaryExpression:})
							/ {:PrimaryExpression: PrimaryExpression:}) -> {}
		PrimaryExpression <- BooleanLiteral / BuiltInCall / RDFLiteral / BrackettedExpression / iriOrFunction  / NumericLiteral /  Var / QName/ LName 
		PrimaryExpression2 <- BooleanLiteral / BuiltInCall / RDFLiteral / BrackettedExpression / iriOrFunction  / NumericLiteral /  Var / LName 
		LName <- ({:LName: Chars_String :}) -> fn_lname
		QName <- ((({:Reference: Reference :} ".") / {:Path: path :}) PrimaryExpression2) -> {}
		Reference <- {Chars_String } => fn_reference
		path <- ((({"INV(" Chars_String ")"} / {"^"? Chars_String} ) ".")^+1) -> {}
		BrackettedExpression <- ('(' space {:BrackettedExpression: Expression :} space ')') -> {}
		BuiltInCall <-  ({:Aggregate: Aggregate :} /  {:FunctionExpression: FunctionExpression :} / {:RegexExpression: RegexExpression :} / {:SubstringExpression: SubstringExpression :} / {:SubstringBifExpression: SubstringBifExpression :} / {:StrReplaceExpression: StrReplaceExpression :} / ExistsFunc / NotExistsFunc) -> {}
		FunctionExpression <- ({:Function: 'STR':} '(' space {:Expression: Expression :} space ')'
			/ ({:Function: 'LANG':} '(' space {:Expression: Expression :} space ')')
			/ ({:Function: 'LANGMATCHES':} '(' space Expression ',' space Expression space ')')
			/ ({:Function: 'DATATYPE':} '(' space {:Expression: Expression :} space ')')
			/ ({:Function: 'BOUND':} '(' space Var space ')')
			/ ({:Function: 'IRI':} '(' space {:Expression: Expression :} space ')')
			/ ({:Function: 'URI':} '(' space {:Expression: Expression :} space ')')
			/ ({:Function: 'BNODE':} ( '(' space {:Expression: Expression :} space ')' / NIL ))
			/ ({:Function: 'RAND':} NIL)
			/ ({:Function: 'ABS':} '(' space {:Expression: Expression :} space ')')
			/ ({:Function: 'CEIL':} '(' space {:Expression: Expression :} space ')')
			/ ({:Function: 'FLOOR':} '(' space {:Expression: Expression :} space ')')
			/ ({:Function: 'ROUND':} '(' space {:Expression: Expression :} space ')')
			/ ({:Function: 'CONCAT':} {:ExpressionList: ExpressionList:})
			/ ({:Function: 'STRLEN':} '(' space {:Expression: Expression :} space ')')
			/ ({:Function: 'UCASE':} '(' space {:Expression: Expression :} space ')')
			/ ({:Function: 'LCASE':} '(' space {:Expression: Expression :} space ')')
			/ ({:Function: 'ENCODE_FOR_URI':} '(' space {:Expression: Expression :} space ')')
			/ ({:Function: 'CONTAINS':} '(' space Expression space ',' space Expression space ')')
			/ ({:Function: 'STRSTARTS':} '(' space Expression space  ',' space Expression space ')')
			/ ({:Function: 'STRENDS':} '(' space Expression space ',' space Expression space ')')
			/ ({:Function: 'STRBEFORE':} '(' space Expression space ',' space Expression space ')')
			/ ({:Function: 'STRAFTER':} '(' space Expression space ',' space Expression space ')')
			/ ({:Function: 'YEAR':} '(' space {:Expression: Expression :} space ')')
			/ ({:Function: 'MONTH':} '(' space {:Expression: Expression :} space ')')
			/ ({:Function: 'DAY':} '(' space {:Expression: Expression :} space ')')
			-- / ({:Function: 'HOURS':} '(' space {:Expression: Expression :} space ')')
			-- / ({:Function: 'MINUTES':} '(' space {:Expression: Expression :} space ')')
			-- / ({:Function: 'SECONDS':} '(' {:Expression: Expression :} space ')')
			/ ({:Function: 'TIMEZONE':} '(' space {:Expression: Expression :} space ')')
			/ ({:Function: 'TZ':} '(' space {:Expression: Expression :} space ')')
			/ ({:Function: 'NOW':} NIL)
			/ ({:Function: 'UUID':} NIL)
			/ ({:Function: 'STRUUID':} NIL)
			/ ({:Function: 'MD5':} '(' space {:Expression: Expression :} space ')')
			/ ({:Function: 'SHA1':} '(' space {:Expression: Expression :} space ')')
			/ ({:Function: 'SHA256':} '(' space {:Expression: Expression :} space ')')
			/ ({:Function: 'SHA384':} '(' space {:Expression: Expression :} space ')')
			/ ({:Function: 'SHA512':} '(' space {:Expression: Expression :} space ')')
			/ ({:Function: 'COALESCE':} {:ExpressionList: ExpressionList:})
			/ ({:Function: 'IF':} '(' space Expression space ',' space Expression ',' space Expression space ')')
			/ ({:Function: 'STRLANG':} '(' space Expression space ',' space Expression space ')')
			/ ({:Function: 'STRDT':} '(' space Expression space ','space  Expression space ')')
			/ ({:Function: 'sameTerm':} '(' space Expression space ',' space Expression space ')')
			/ ({:Function: 'isIRI':} '(' space {:Expression: Expression :} space ')')
			/ ({:Function: 'isURI':} '(' space {:Expression: Expression :} space ')')
			/ ({:Function: 'isBLANK':} '(' space {:Expression: Expression :} space ')')
			/ ({:Function: 'dateTime':} '(' space {:Expression: Expression :} space ')')
			/ ({:FunctionTime: 'days':} '(' space {:PrimaryExpressionL: PrimaryExpression :} "-" {:PrimaryExpressionR: PrimaryExpression :} space ')')
			/ ({:FunctionTime: 'years':} '(' space {:PrimaryExpressionL: PrimaryExpression :} "-" {:PrimaryExpressionR: PrimaryExpression :} space ')')
			/ ({:FunctionTime: 'months':} '(' space {:PrimaryExpressionL: PrimaryExpression :} "-" {:PrimaryExpressionR: PrimaryExpression :} space ')')
			/ ({:FunctionTime: 'hours':} '(' space {:PrimaryExpressionL: PrimaryExpression :} "-" {:PrimaryExpressionR: PrimaryExpression :} space ')')
			/ ({:FunctionTime: 'minutes':} '(' space {:PrimaryExpressionL: PrimaryExpression :} "-" {:PrimaryExpressionR: PrimaryExpression :} space ')')
			/ ({:FunctionTime: 'seconds':} '(' space {:PrimaryExpressionL: PrimaryExpression :} "-" {:PrimaryExpressionR: PrimaryExpression :} space ')')
			/ ({:FunctionTime: ('date'):} '(' space {:Expression: Expression :} space ')')
			/ ({:Function: 'isLITERAL':} '(' space {:Expression: Expression :} space ')')
			/ ({:Function: 'isNUMERIC':} '(' space {:Expression: Expression :} space ')')) -> {}
		iriOrFunction <- ({:iri: iri:} {:ArgList: ArgList?:}) -> {}
		ArgList <- ('(' space {:DISTINCT: 'DISTINCT'? :} space {:ArgListExpression: ArgListExpression :} space ')' / NIL ) -> {}
		ArgListExpression <-  (Expression ( ',' space Expression )*) -> {}
		RDFLiteral <- 	({:RDFLiteral: RDFLiteralA:}) -> {}
		RDFLiteralA <- 	({String} ( {:LANGTAG: LANGTAG:} / ( '^^' {:iri: iri:} ) )?) -> {}
		NumericLiteral <-	({:NumericLiteral: NumericLiteralUnsigned / NumericLiteralPositive / NumericLiteralNegative :})->{}
		NumericLiteralUnsigned <-  DECIMAL / DOUBLE / INTEGER
		NumericLiteralPositive <- DECIMAL_POSITIVE / DOUBLE_POSITIVE / INTEGER_POSITIVE
		NumericLiteralNegative <- DECIMAL_NEGATIVE / DOUBLE_NEGATIVE / INTEGER_NEGATIVE
		BooleanLiteral <- ({:BooleanLiteral: 'true' / 'false':}) -> {}
		Var <- ({:var: VAR1 / VAR2 :}) ->{}
		VAR1 <- {'?' VARNAME}
		VAR2 <- {'$' VARNAME}
		-- VARNAME <- ({([A-Za-z] / "_") ([A-Za-z] / "_" / [0-9])*})
		ExpressionList <- (NIL / '(' space Expression space  ( ',' space Expression )* space ')' ) -> {}
		ExpressionList2 <- (NIL / '(' space Expression space  ( ',' space Expression )* space ')' )
		RegexExpression <- ('REGEX' '(' space Expression space  ',' space Expression ( ',' space Expression )? space ')') -> {}
		SubstringExpression <- (('SUBSTRING' / 'SUBSTR' ) '(' space Expression space  ',' space Expression ( ',' space Expression )? space ')') -> {}
		SubstringBifExpression <- (('bif:SUBSTRING' / 'bif:SUBSTR' ) '(' space Expression space  ',' space Expression ( ',' space Expression )? space ')') -> {}
		StrReplaceExpression <- ('REPLACE' '(' space Expression space  ',' space Expression ',' space Expression ( ',' space Expression )? space ')') ->{}
		ExistsFunc <- ({:ExistsFunc: 'EXISTS' space GroupGraphPattern :} / {:ExistsFunc: '{' space GroupGraphPattern space '}' :})
		NotExistsFunc <- ( ('NOT' space  '{' space {:NotExistsFunc: GroupGraphPattern :} space '}' ) / {:NotExistsFunc: 'NOT' space  'EXISTS'? space GroupGraphPattern :} )
		-- GroupGraphPattern <- ( {:PrimaryExpression: PrimaryExpression :}) -> {}
		GroupGraphPattern <- ( {:Expression: Expression :}) -> {}
		Aggregate <- (({:Aggregate: 'COUNT' :} '(' space {:DISTINCT: 'DISTINCT'? :} space ( {'*'} / {:Expression: Expression :} ) space ')')
					/ ({:Aggregate: 'SUM' :} '(' space {:DISTINCT: 'DISTINCT'? :} space {:Expression: Expression :} space ')')
					/ ({:Aggregate: 'MIN' :} '(' space {:DISTINCT: 'DISTINCT'? :} space {:Expression: Expression :} space ')')
					/ ({:Aggregate: 'MAX' :} '(' space {:DISTINCT: 'DISTINCT'? :} space{:Expression: Expression :} space ')')
					/ ({:Aggregate: 'AVG' :} '(' space {:DISTINCT: 'DISTINCT'? :} space {:Expression: Expression :} space ')')
					/ ({:Aggregate: 'SAMPLE' :} '(' space {:DISTINCT: 'DISTINCT'? :} space{:Expression: Expression :} space ')')
					/ ({:Aggregate: 'GROUP_CONCAT' :} '(' space {:DISTINCT: 'DISTINCT'? :} space{:Expression: Expression :} ( ';' space 'SEPARATOR' '=' {:SEPARATOR: String :} )? space ')')) -> {}
		iri <- ({:IRIREF: IRIREF:} / {:PrefixedName: PrefixedName:}) -> {}
		PrefixedName <- {PNAME_LN} / {PNAME_NS}
		IRIREF <- '<' ([A-Za-z] / "_" / ":" / "." / "#" / "/" / [0-9])* '>'
		-- IRIREF <- '<' ([^<>"{}|^`\]-[#x00-#x20])* '>'
		PNAME_NS <- PN_PREFIX? ':'
		PNAME_LN <- PNAME_NS PN_LOCAL
		String <- STRING_LITERAL1 / STRING_LITERAL2 / STRING_LITERAL_LONG1 / STRING_LITERAL_LONG2
		-- STRING_LITERAL1 <- "'" ( ([^#x27#x5C#xA#xD]) / ECHAR )* "'"
		STRING_LITERAL1 <- "'" ( ([A-Za-z] / [-_.: ] / [0-9]) / ECHAR )* "'"
		-- STRING_LITERAL2 <- '"' ( ([^#x22#x5C#xA#xD]) / ECHAR )* '"'
		STRING_LITERAL2 <- '"' (  ([A-Za-z] / [-_.: ] / [0-9]) / ECHAR )* '"'
		STRING_LITERAL_LONG1 <- "'''" ( ( "'" / "''" )? ( [^'\] / ECHAR ) )* "'''"
		STRING_LITERAL_LONG2 <- '"""' ( ( '"' / '""' )? ( [^"\] / ECHAR ) )* '"""'
		ECHAR <- '\' [tbnrf\"']
		NIL <- '('  ')'
		WS <- ({([A-Za-z] / "_") ([A-Za-z] / "_" / [0-9])*})
		--WS <- #x20 / #x9 / #xD / #xA
		LANGTAG <- ({'@' [A-Za-z] ([A-Za-z])* ('-' [a-zA-Z0-9]+)*})
		INTEGER	  <-  	[0-9]+
    	DECIMAL	  <-  	([0-9]* '.' [0-9]+)
		DOUBLE	  <-  	([0-9]+ '.' [0-9]* EXPONENT) / ('.' ([0-9])+ EXPONENT) / (([0-9])+ EXPONENT)
		INTEGER_POSITIVE	  <-  	'+' INTEGER
		DECIMAL_POSITIVE	  <-  	'+' DECIMAL
		DOUBLE_POSITIVE	  <- 	'-' INTEGER
		INTEGER_NEGATIVE	  <-  	'-' INTEGER
		DECIMAL_NEGATIVE	  <-  	'-' DECIMAL
		DOUBLE_NEGATIVE	  <- 	'-' DOUBLE
		EXPONENT	  <-  	[eE] [+-]? [0-9]+
		-- PN_CHARS_BASE <- [A-Z] / [a-z] / [#x00C0-#x00D6] / [#x00D8-#x00F6] / [#x00F8-#x02FF] / [#x0370-#x037D] / [#x037F-#x1FFF] / [#x200C-#x200D] / [#x2070-#x218F] / [#x2C00-#x2FEF] / [#x3001-#xD7FF] / [#xF900-#xFDCF] / [#xFDF0-#xFFFD] / [#x10000-#xEFFFF]
		PN_CHARS_BASE <- ()[A-Z] / [a-z] 
		PN_CHARS_U <- PN_CHARS_BASE / '_'
		-- VARNAME	 <- 	( PN_CHARS_U / [0-9] ) ( PN_CHARS_U / [0-9] / #x00B7 / [#x0300-#x036F] / [#x203F-#x2040] )*
		VARNAME	 <- 	( PN_CHARS_U / [0-9] ) ( PN_CHARS_U / [0-9] /  [A-Z] / [a-z])*
		-- PN_CHARS <- 	PN_CHARS_U / '-' / [0-9] / #x00B7 / [#x0300-#x036F] / [#x203F-#x2040]
		PN_CHARS <- 	PN_CHARS_U / '-' / [0-9] / [A-Z] / [a-z]
		-- PN_PREFIX <- PN_CHARS_BASE ((PN_CHARS/'.')* PN_CHARS)?
		PN_PREFIX <- Chars_String
		-- PN_LOCAL <- (PN_CHARS_U / ':' / [0-9] / PLX ) ((PN_CHARS / '.' / ':' / PLX)* (PN_CHARS / ':' / PLX) )?
		PN_LOCAL <- Chars_String
		PLX	 <- PERCENT / PN_LOCAL_ESC
		PERCENT	 <- '%' HEX HEX
		HEX <- [0-9] / [A-F] / [a-f]
		PN_LOCAL_ESC <- '\' ( '_' / '~' / '.' / '-' / '!' / '$' / '&' / "'" / '(' / ')' / '*' / '+' / ',' / ';' / '=' / '/' / '?' / '#' / '@' / '%' )
		Chars_String <- (([A-Za-z] / "_") ([A-Za-z] / "_" / [0-9])*)
		space <- ((" ")*)
	]]
	,{
		fn_lname = function (capture, current_pos) 
			-- triplesSameSubjectTable = {}
			triplesSameSubjectTable[capture] = {
				  class = className,
				  prefixedName = capture,
				  var = capture
				}
			if capture == classInProc:find("/compartment:has(/compartType[id='Name'])"):attr("value") then
				return {var = "?"..idTable[classInProc:id()]}
			-- elseif lQuery("Onto#AllAssociation[name='" .. capture .. "']"):is_not_empty() then 
				-- return {var = ":"..capture}
			else
				return {var = "?"..capture}
			end
		end,
		fn_reference = function(allString, current_pos, capture)
			diagram = utilities.current_diagram()
			--atrast id
			--ja diagramma ir node ar tadu id
			local node = nil
			for key, value in pairs(idTable) do
				if value == capture then node = diagram:find("/element.Node"):filter(function(n) return n:id() == tonumber(key) end) end
			end	
			if node ~= nil then
			-- if diagram:find("/element.Node/compartment:has(/compartType[id='Instance'])[value='" .. capture .. "']"):is_not_empty() then
				return current_pos
			end
			return false
		end,
	})
	
	return grammar
end

function isValidForConvertation(v)
	if isDateVar(v) == true then
		return true
	else
		if v["RDFLiteral"]~=nil then
			local value = v["RDFLiteral"]
			if value["iri"]==nil then
				return true
			else return false end
		end
	end
	return false
end

function isDateTimeVar(v)
	--var
	if v["var"]~=nil then 
		local value = v["var"]
		if lQuery("Onto#Attribute[name='" .. string.sub(value, 2) .."']"):is_not_empty() and lQuery("Onto#Attribute[name='" .. string.sub(value, 2) .."']"):attr("type") == "XSD_DATE_TIME" then return true
		else return false end
	end
	--RDFLiteral
	if v["RDFLiteral"]~=nil then
		local value = v["RDFLiteral"]
		if value["iri"]~=nil and value["iri"]["PrefixedName"]~=nil and string.lower(value["iri"]["PrefixedName"]) == "xsd:datetime" then
			return true
		else return false end
	end
	--iri
	if v["iri"]~=nil and v["iri"]["PrefixedName"]~=nil and string.lower(v["iri"]["PrefixedName"]) == "xsd:datetime" then
		return true
	else return false end
	return false
end

function isDateVar(v)
	--var
	if v["var"]~=nil then 
		local value = v["var"]
		if lQuery("Onto#Attribute[name='" .. string.sub(value, 2) .."']"):is_not_empty() and lQuery("Onto#Attribute[name='" .. string.sub(value, 2) .."']"):attr("type") == "XSD_DATE" then return true
		else return false end
	end
	--RDFLiteral
	if v["RDFLiteral"]~=nil then
		local value = v["RDFLiteral"]
		if value["iri"]~=nil and value["iri"]["PrefixedName"]~=nil and string.lower(value["iri"]["PrefixedName"]) == "xsd:date" then
			return true
		else return false end
	end
	--iri
	if v["iri"]~=nil and v["iri"]["PrefixedName"]~=nil and string.lower(v["iri"]["PrefixedName"]) == "xsd:date" then
		return true
	else return false end
	return false
end

function getSPARQLGpoupGraphPatternExpression(expressionTable, SPARQLstring)
	
	local toolType = lQuery("ToolType")
	if toolType:find("/tag[key = 'counter']"):is_empty() then lQuery.create("Tag", {key = 'counter', value = 0}):link("thing", toolType) end
	
		-- toolType:find("/tag[key = 'counter']"):attr("value", tonumber(toolType:find("/tag[key = 'counter']"):attr("value"))+1)
	for key, value in pairs(expressionTable) do
		-- print(key, value)
		local visited = 0
		if key == "Path" then pathInString = true end
		if key == "PrimaryExpression" and value["Reference"]~=nil then
			local varName = string.sub(value[1]["var"], 2)
			triplesSameSubjectTable[varName] = {
				class = value["Reference"],
				prefixedName = varName,
				var = varName .. "_" .. value["Reference"]
			}
			classTriplesTable[value["Reference"]] = value["Reference"]
			-- visited = 1
			value[1]["var"] = value[1]["var"] .. "_" .. value["Reference"]
		end
		
		if key == "GpoupGraphPattern" then
			SPARQLstring = SPARQLstring .. "{" .. getSPARQLGpoupGraphPatternExpression(value, "") .. "}"
			visited = 1
		end
		
		if key == "BrackettedExpression" then
			SPARQLstring = SPARQLstring .. "(" .. getSPARQLGpoupGraphPatternExpression(value, "") .. ")"
			visited = 1
		end
		
		if key == "TriplesSameSubject" then
			local varName = value["propertyListNotEmpty"]["objectList"]["object"]["graphNode"]["varOrTerm"]["var"]
			
			diagram = utilities.current_diagram()
			--find central class

			local collection = ""
			if diagram:find("/collection/element"):is_not_empty() then collection = "/collection" end
			
			startClass = diagram:find(collection .. "/element:has(/elemType[id = 'Class'])"):filter(function(class)
				return class:find("/compartment:has(/compartType[id='ClassType'])"):attr("value") == "query"
			end)
			
			-- if "?"..startClass:find("/compartment:has(/compartType[id='Instance'])"):attr("value") ~= value["varOrTerm"]["var"] then
			if "?"..idTable[startClass:id()] ~= value["varOrTerm"]["var"] then
				if attributesNamesTable[varName] == nil then 
					if 	attributesNamesTableP[varName] == nil then
						attributesNamesTable[varName] = varName
					else
						attributesNamesTable[varName] = varName .. "_" .. toolType:find("/tag[key = 'counter']"):attr("value")	
						toolType:find("/tag[key = 'counter']"):attr("value", tonumber(toolType:find("/tag[key = 'counter']"):attr("value"))+1)
					end
				
				end
			else
				attributesNamesTable[varName] = varName
			end
			
			varName = attributesNamesTable[varName]	
			SPARQLstring = SPARQLstring .. value["varOrTerm"]["var"] .. " " .. value["propertyListNotEmpty"]["verb"]["varOrIRIref"]["iriRef"]["prefixedName"] .. " " .. varName .. ".\n"
			if value["TriplesBlock"]~=nil then
				SPARQLstring = SPARQLstring  .. getSPARQLGpoupGraphPatternExpression(value["TriplesBlock"], "")
			end
			visited = 1
		end
		
		if key == "Filter" then
			SPARQLstring = SPARQLstring .. "FILTER " .. getSPARQLGpoupGraphPatternExpression(value, "") 
			visited = 1
		end
		if key == "NotExistsFunc" then
			SPARQLstring = SPARQLstring .. "NOT EXISTS " .. getSPARQLGpoupGraphPatternExpression(value, "")
			visited = 1
		end
		
		if key == "BooleanLiteral" then
			SPARQLstring = SPARQLstring .. tostring(value)
		end
		
		if key == "var" then
			if attributesNamesTable[value] == nil then 
				if lQuery("Onto#Attribute[name='" .. string.sub(value, 2) .."']"):is_not_empty() or lQuery("Onto#AllAssociation[name='" .. string.sub(value, 2) .."']"):is_not_empty() then
					if 	attributesNamesTableP[value] == nil then
						attributesNamesTable[value] = value
					else
						attributesNamesTable[value] = value .. "_" .. toolType:find("/tag[key = 'counter']"):attr("value")	
						toolType:find("/tag[key = 'counter']"):attr("value", tonumber(toolType:find("/tag[key = 'counter']"):attr("value"))+1)
					end
				end
			end
			if attributesNamesTable[value] ~= nil then value = attributesNamesTable[value] end
			SPARQLstring = SPARQLstring .. value
		end
		
		if key == "Additive" or key == "Unary" then
			isFunction = true
			SPARQLstring = SPARQLstring .. value
		end
		
		if key == "RDFLiteral" then
			SPARQLstring = SPARQLstring .. value[1]
			if value["LANGTAG"]~= nil then
				SPARQLstring = SPARQLstring .. value["LANGTAG"]
			end
			if value["iri"]~=nil then
				SPARQLstring = SPARQLstring .. "^^"
				if value["iri"]["PrefixedName"]~=nil then
					SPARQLstring = SPARQLstring  .. value["iri"]["PrefixedName"]
				end
				if value["iri"]["IRIREF"]~=nil then
					SPARQLstring = SPARQLstring  .. value["iri"]["IRIREF"]
				end
			end
		end
		
		if key == "Aggregate" and type(value) == "table" then
			isAggregate = true
			SPARQLstring = SPARQLstring .. value["Aggregate"]
			SPARQLstring = SPARQLstring .. "(" .. value["DISTINCT"] .. " " .. getSPARQLGpoupGraphPatternExpression(value, "") .. ")"
			visited = 1
		end
		
		if key == "RelationalExpression" then
			SPARQLstring = SPARQLstring  .. getSPARQLGpoupGraphPatternExpression(value["NumericExpressionL"], "") 
			if value["Relation"]~= nil then 
				if value["Relation"] == "<>" then SPARQLstring = SPARQLstring  .. " !=" 
				else SPARQLstring = SPARQLstring  .. " " .. value["Relation"] end
				if value["Relation"] == "IN" or value["Relation"] == "NOT IN" then
					SPARQLstring = SPARQLstring .. "(" .. getSPARQLGpoupGraphPatternExpression2(value, "") .. ")" 
				end
				if value["NumericExpressionR"]~= nil then
					SPARQLstring = SPARQLstring  .. getSPARQLGpoupGraphPatternExpression(value["NumericExpressionR"], "") 
				end
			end
			visited = 1
		end
		
		if key == "NumericLiteral" then
			SPARQLstring = SPARQLstring .. value
		end
		
		if key == "ConditionalOrExpression" then
			for k,v in pairs (value) do
				SPARQLstring = SPARQLstring  .. getSPARQLGpoupGraphPatternExpression(v, "") 
				if k ~= #value then
					SPARQLstring = SPARQLstring  ..  " || "
				end
			end
			visited = 1
		end
		
		if key == "ConditionalAndExpression" then
			for k,v in pairs (value) do
				SPARQLstring = SPARQLstring  .. getSPARQLGpoupGraphPatternExpression(v, "") 
				if k ~= #value then
					SPARQLstring = SPARQLstring  ..  " && "
				end
			end
			visited = 1
		end
		
		if key == "GroupOrUnionGraphPattern" then
			for k,v in pairs (value) do
				SPARQLstring = SPARQLstring  .. getSPARQLGpoupGraphPatternExpression(v, "") 
				if k ~= #value then
					SPARQLstring = SPARQLstring  ..  " UNION "
				end
			end
			visited = 1
		end
		
		if key == "MultiplicativeExpression" then
			if value["UnaryExpression"]["Additive"]~= nil then
				SPARQLstring = SPARQLstring  .. value["UnaryExpression"]["Additive"]
				SPARQLstring = SPARQLstring  .. getSPARQLGpoupGraphPatternExpression(value["UnaryExpression"]["PrimaryExpression"], "")
			else
				SPARQLstring = SPARQLstring  .. getSPARQLGpoupGraphPatternExpression(value["UnaryExpression"], "")
			end
			
			if value["UnaryExpressionList"]~= nil then
				SPARQLstring = SPARQLstring  .. getSPARQLGpoupGraphPatternExpression(value["UnaryExpressionList"], "")
			end
			visited = 1
		end
		
		if key == "MultiplicativeExpressionList" then
			for k,v in pairs (value) do
				if v["Additive"]~= nil then
					SPARQLstring = SPARQLstring  .. v["Additive"]
					SPARQLstring = SPARQLstring  .. getSPARQLGpoupGraphPatternExpression({MultiplicativeExpression = v["MultiplicativeExpression"]}, "")
					
				end
			end
			visited = 1
			-- if value["Additive"]~= nil then
				-- SPARQLstring = SPARQLstring  .. value["Additive"]
				-- SPARQLstring = SPARQLstring  .. getSPARQLGpoupGraphPatternExpression({MultiplicativeExpression = value["MultiplicativeExpression"]}, "")
				-- visited = 1
			-- end
		end
		
		if key == "AdditiveExpression" then
			if value["MultiplicativeExpressionList"]~= nil and value["MultiplicativeExpressionList"][1]~=nil and ((value["MultiplicativeExpressionList"][1]["Concat"]~= nil) or (value["MultiplicativeExpressionList"][2] == nil and value["MultiplicativeExpressionList"][1]["Additive"]~=nil and value["MultiplicativeExpressionList"][1]["Additive"]=="-") ) then
			else SPARQLstring = SPARQLstring  .. getSPARQLGpoupGraphPatternExpression({MultiplicativeExpression = value["MultiplicativeExpression"]}, "") end
			if value["MultiplicativeExpressionList"]~= nil then
				if value["MultiplicativeExpressionList"][1]~=nil and value["MultiplicativeExpressionList"][1]["Concat"]~= nil then
					local concat = " CONCAT(" .. getSPARQLGpoupGraphPatternExpression({MultiplicativeExpression = value["MultiplicativeExpression"]}, "")
					for k,v in pairs(value["MultiplicativeExpressionList"]) do
						if v["Concat"]~=nil then
							isFunction = true
							concat = concat .. ", " .. getSPARQLGpoupGraphPatternExpression({MultiplicativeExpression = v["MultiplicativeExpression"]}, "")
						else break
						end
					end
					concat = concat .. ") "
					SPARQLstring = SPARQLstring .. concat
				elseif value["MultiplicativeExpressionList"][1]~=nil and value["MultiplicativeExpressionList"][2] == nil and value["MultiplicativeExpressionList"][1]["Additive"]~=nil and value["MultiplicativeExpressionList"][1]["Additive"]=="-" then
					if value["MultiplicativeExpression"]~=nil and
					   value["MultiplicativeExpression"]["UnaryExpression"]~=nil and
					   (value["MultiplicativeExpression"]["UnaryExpressionList"]==nil or (value["MultiplicativeExpression"]["UnaryExpressionList"]~=nil and value["MultiplicativeExpression"]["UnaryExpressionList"][1]==nil)) and
					   value["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]~=nil and
						
					   value["MultiplicativeExpressionList"][1]["MultiplicativeExpression"]~=nil and
					   value["MultiplicativeExpressionList"][1]["MultiplicativeExpression"]["UnaryExpression"]~=nil and
					   (value["MultiplicativeExpressionList"][1]["MultiplicativeExpression"]["UnaryExpressionList"]==nil or (value["MultiplicativeExpressionList"][1]["MultiplicativeExpression"]["UnaryExpressionList"]~=nil and value["MultiplicativeExpressionList"][1]["MultiplicativeExpression"]["UnaryExpressionList"][1]==nil)) and
					   value["MultiplicativeExpressionList"][1]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]~=nil then
						local left = value["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]
						local right = value["MultiplicativeExpressionList"][1]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]
					
						local sl = getSPARQLGpoupGraphPatternExpression({PrimaryExpression = left}, "")
						local sr = getSPARQLGpoupGraphPatternExpression({PrimaryExpression = right}, "")
						if isDateVar(left) == true and isDateVar(right) == true then
							SPARQLstring = SPARQLstring .. 'bif:datediff("day", ' .. sr .. ", " .. sl .. ")"
							isFunction = true
						elseif isDateTimeVar(left) == true and isDateTimeVar(right) == true then
							SPARQLstring = SPARQLstring .. 'bif:datediff("day", ' .. sr .. ", " .. sl .. ")"
							isFunction = true
						elseif isDateTimeVar(left) == true and isValidForConvertation(right) then
							SPARQLstring = SPARQLstring .. 'bif:datediff("day", xsd:dateTime(' .. sr .. '),' .. sl .. ")"
							isFunction = true
						elseif isDateTimeVar(left) == true then
							SPARQLstring = SPARQLstring .. 'bif:datediff("day",  ' .. sr .. ', xsd:dateTime(' .. sl .. "))"
							isFunction = true
						else
							SPARQLstring = SPARQLstring  .. getSPARQLGpoupGraphPatternExpression({MultiplicativeExpression = value["MultiplicativeExpression"]}, "")
							SPARQLstring = SPARQLstring  .. getSPARQLGpoupGraphPatternExpression({MultiplicativeExpressionList = value["MultiplicativeExpressionList"]}, "")
						end
					end
				else
					SPARQLstring = SPARQLstring  .. getSPARQLGpoupGraphPatternExpression({MultiplicativeExpressionList = value["MultiplicativeExpressionList"]}, "")
				end
			end
			visited = 1
		end
		
		if key == "PrimaryExpression" and value["iri"]~=nil then
			if value["iri"]["PrefixedName"]~=nil then
				local valueString = value["iri"]["PrefixedName"]
				if valueString == "vq:datediff" then valueString = "bif:datediff" 
				elseif string.lower(valueString) == "xsd:date" or string.lower(valueString) == "xsd:datetime" then isFunction = true
				else
					if lQuery("Onto#Prefix[shortForm='" .. string.sub(valueString,1, string.find(valueString, ":")) .. "']"):is_not_empty()
					and value["ArgList"]~=nil and value["ArgList"]==""	then
						prefixes[string.sub(valueString,1, string.find(valueString, ":"))] = lQuery("Onto#Prefix[shortForm='" .. string.sub(valueString,1, string.find(valueString, ":")) .. "']"):attr("fullForm")
						valueString = "?" .. string.sub(valueString, string.find(valueString, ":")+1)
					end
				end
				
				SPARQLstring = SPARQLstring  .. valueString
			end
			if value["iri"]["IRIREF"]~=nil then
				SPARQLstring = SPARQLstring  .. value["iri"]["IRIREF"]
			end
			if value["ArgList"]~=nil and value["ArgList"]~="" then
				SPARQLstring = SPARQLstring  .. "(" .. value["ArgList"]["DISTINCT"] .. " " .. getSPARQLGpoupGraphPatternExpression({ArgList = value["ArgList"]}, "") .. ")"
			end
			visited = 1
		end
		
		if key == "FunctionExpression" then
			isFunction = true
			if value["Function"]~=nil then
				SPARQLstring = SPARQLstring  .. value["Function"]
			end
			if value["Expression"]~=nil then
				SPARQLstring = SPARQLstring  .. "(" .. getSPARQLGpoupGraphPatternExpression({Expression = value["Expression"]}, "") .. ")"
			end
			if value["ExpressionList"]~=nil then
				SPARQLstring = SPARQLstring  .. "(" .. getSPARQLGpoupGraphPatternExpression({ExpressionList = value["ExpressionList"]}, "") .. ")"
			end
			
			if value["FunctionTime"]~=nil then
				if lQuery("Onto#Parameter[name='SPARQL engine type']"):attr("value") == "OpenLink Virtuoso" then
					local fun = value["FunctionTime"]
					local sl = getSPARQLGpoupGraphPatternExpression({PrimaryExpression = value["PrimaryExpressionL"]}, "")
					local sr = getSPARQLGpoupGraphPatternExpression({PrimaryExpression = value["PrimaryExpressionR"]}, "")
					if isDateVar(value["PrimaryExpressionL"]) == true and isDateVar(value["PrimaryExpressionR"]) == true then
						SPARQLstring = SPARQLstring .. 'bif:datediff("' .. string.sub(fun, 1, string.len(fun)-1) .. '", ' .. sr .. ", " .. sl .. ")"
					elseif isDateTimeVar(value["PrimaryExpressionL"]) == true and isDateTimeVar(value["PrimaryExpressionR"]) == true then
						SPARQLstring = SPARQLstring .. 'bif:datediff("' .. string.sub(fun, 1, string.len(fun)-1) .. '", ' .. sr .. ", " .. sl .. ")"
					elseif isDateTimeVar(value["PrimaryExpressionL"]) == true and isValidForConvertation(value["PrimaryExpressionR"]) then
						-- SPARQLstring = SPARQLstring .. 'bif:datediff("' .. string.sub(fun, 1, string.len(fun)-1) .. '", ' .. sl .. ", xsd:dateTime(" .. sr .. "))"
						SPARQLstring = SPARQLstring .. 'bif:datediff("' .. string.sub(fun, 1, string.len(fun)-1) .. ',  xsd:dateTime(' .. sr .. "), " .. sl .. ")"
					elseif isDateTimeVar(value["PrimaryExpressionR"]) == true then
						-- SPARQLstring = SPARQLstring .. 'bif:datediff("' .. string.sub(fun, 1, string.len(fun)-1) .. ',  xsd:dateTime(' .. sl .. "), " .. sr .. ")"
						SPARQLstring = SPARQLstring .. 'bif:datediff("' .. string.sub(fun, 1, string.len(fun)-1) .. '", ' .. sr .. ", xsd:dateTime(" .. sl .. "))"
					else
						incorrectSyntaxForm(fun .. "(" .. sl .. "-" .. sr .. ")")
						SPARQLstring = SPARQLstring .. fun .. "(" .. sl .. "-" .. sr .. ")"
					end
				else
					local s = value["FunctionTime"] .. "(" .. getSPARQLGpoupGraphPatternExpression({PrimaryExpression = value["PrimaryExpressionL"]}, "") ..  "-" .. getSPARQLGpoupGraphPatternExpression({PrimaryExpression = value["PrimaryExpressionR"]}, "") .. ")"
					incorrectSyntaxForm(s)
					SPARQLstring = SPARQLstring  .. s
				end
			end

			visited = 1
		end
		
		
		if key == "ExpressionList" then
			for k,v in pairs (value) do
				SPARQLstring = SPARQLstring  .. getSPARQLGpoupGraphPatternExpression(v, "") 
				if k ~= #value then
					SPARQLstring = SPARQLstring  ..  ", "
				end
			end
			visited = 1
		end
		
		if key == "ArgListExpression" then
			for k,v in pairs (value) do
				SPARQLstring = SPARQLstring  .. getSPARQLGpoupGraphPatternExpression(v, "") 
				if k ~= #value then
					SPARQLstring = SPARQLstring  ..  ", "
				end
			end
			visited = 1
		end
		
		if key == "RegexExpression" then
			SPARQLstring = SPARQLstring  .. "REGEX(" .. getSPARQLGpoupGraphPatternExpression({ExpressionList = value}, "") .. ")"
			visited = 1
		end
		
		if key == "SubstringExpression" then
			isFunction = true
			if lQuery("Onto#Parameter[name='SPARQL engine type']"):attr("value") == "OpenLink Virtuoso" then
				SPARQLstring = SPARQLstring  .. "bif:substring(" .. getSPARQLGpoupGraphPatternExpression({ExpressionList = value}, "") .. ")"
			else
				SPARQLstring = SPARQLstring  .. "SUBSTR(" .. getSPARQLGpoupGraphPatternExpression({ExpressionList = value}, "") .. ")"
			end
			visited = 1
		end
		
		if key == "SubstringBifExpression" then
			isFunction = true
			SPARQLstring = SPARQLstring  .. "bif:substring(" .. getSPARQLGpoupGraphPatternExpression({ExpressionList = value}, "") .. ")"
			visited = 1
		end
		
		if key == "StrReplaceExpression" then
			SPARQLstring = SPARQLstring  .. "REPLACE(" .. getSPARQLGpoupGraphPatternExpression({ExpressionList = value}, "") .. ")"
			visited = 1
		end
		
		if visited == 0 and type(value) == "table" then
			SPARQLstring = getSPARQLGpoupGraphPatternExpression(value, SPARQLstring)
		end
	end
	return SPARQLstring
end

function getSPARQLGpoupGraphPatternExpression2(expressionTable, SPARQLstring)
	for key, value in pairs(expressionTable) do
		local visited = 0
		if key == "NumericLiteral" then
			if SPARQLstring == "" then
				SPARQLstring = value
			else
				SPARQLstring = SPARQLstring .. ", ".. value 
			end
		end
		if key == "RDFLiteral" then
			if SPARQLstring == "" then
				SPARQLstring = value[1]
			else
				SPARQLstring = SPARQLstring .. ", ".. value[1] 
			end
		end
		if visited == 0 and type(value) == "table" then
			SPARQLstring = getSPARQLGpoupGraphPatternExpression2(value, SPARQLstring)
		end
	end
	
	return SPARQLstring
end

function getSPARQLrecurtuion4(expressionTable)
	for key, value in pairs(expressionTable) do
		if key == "GpoupGraphPattern" and value["Filters"]~=nil 
			and value["Filters"][1]~=nil and value["Filters"][2]==nil and value["Filters"][1]["GroupOrUnionGraphPattern"]~=nil
			and value["Filters"][1]["GroupOrUnionGraphPattern"][1]~=nil
			and value["Filters"][1]["GroupOrUnionGraphPattern"][2]==nil
			and value["Filters"][1]["GroupOrUnionGraphPattern"][1]["GpoupGraphPattern"]~=nil
			then
				expressionTable["GpoupGraphPattern"] = value["Filters"][1]["GroupOrUnionGraphPattern"][1]["GpoupGraphPattern"]
		end
		
		if (key == "RelationalExpression" and (value["Relation"] == "IN" or value["Relation"] == "NOT IN")) or key=="Aggregate" or key == "FunctionExpression"
		or key == "ArgListExpression" or key == "RegexExpression" or key == "SubstringExpression" or key == "SubstringBifExpression" or key=="StrReplaceExpression"  or key == "ExpressionList" then
			return expressionTable
		end
		
		if type(value) == "table" then --and key~= "NotExistsFunc" then
			getSPARQLrecurtuion4(value)
		end
		if key == "RelationalExpression" --and value["Relation"] ==nil
		 and value["NumericExpressionL"]~=nil and value["NumericExpressionR"] == nil 
		 and value["NumericExpressionL"]["AdditiveExpression"]~= nil and (value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpressionList"]==nil or (value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpressionList"]~=nil and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpressionList"][1]==nil))
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]~=nil and (value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpressionList"]==nil or (value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpressionList"]~=nil and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpressionList"][1]==nil))
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]~=nil and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["Additive"]==nil
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]~=nil and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["Relation"] == nil
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["var"] ~= nil
		 then
		 	value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"] = {BooleanLiteral = true}
		end
		--DateTime-------------------------
		if key == "RelationalExpression" and value["Relation"] ~=nil 
		 and value["NumericExpressionL"]~=nil 
		 and value["NumericExpressionL"]["AdditiveExpression"]~= nil
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]~=nil
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]~=nil
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]~=nil
		 and ((value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["var"] ~= nil
		 and lQuery("Onto#Attribute[name='" .. string.sub(value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["var"], 2) .. "']"):is_not_empty()
		 and lQuery("Onto#Attribute[name='" .. string.sub(value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["var"], 2) .. "']"):attr("type") == "XSD_DATE_TIME")
		 or (value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["iri"] ~= nil
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["iri"]["PrefixedName"] ~= nil
		 and string.lower(value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["iri"]["PrefixedName"]) == "xsd:datetime"
		 ))
		 and value["NumericExpressionR"] ~= nil 
		 and value["NumericExpressionR"]["AdditiveExpression"]~= nil
		 and value["NumericExpressionR"]["AdditiveExpression"]["MultiplicativeExpression"]~=nil
		 and value["NumericExpressionR"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]~=nil
		 and value["NumericExpressionR"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]~=nil
		 and value["NumericExpressionR"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["RDFLiteral"]~=nil
		 and value["NumericExpressionR"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["RDFLiteral"][1]~=nil
		then
			value["NumericExpressionR"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["RDFLiteral"][1] = "xsd:dateTime(" .. value["NumericExpressionR"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["RDFLiteral"][1] .. ")"
		end
		
		if key == "RelationalExpression" and value["Relation"] ~=nil 
		 and value["NumericExpressionR"]~=nil 
		 and value["NumericExpressionR"]["AdditiveExpression"]~= nil
		 and value["NumericExpressionR"]["AdditiveExpression"]["MultiplicativeExpression"]~=nil
		 and value["NumericExpressionR"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]~=nil
		 and value["NumericExpressionR"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]~=nil
		 and ((value["NumericExpressionR"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["var"] ~= nil
		 and lQuery("Onto#Attribute[name='" .. string.sub(value["NumericExpressionR"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["var"], 2) .. "']"):is_not_empty()
		 and lQuery("Onto#Attribute[name='" .. string.sub(value["NumericExpressionR"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["var"], 2) .. "']"):attr("type") == "XSD_DATE_TIME")
		 or (value["NumericExpressionR"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["iri"] ~= nil
		 and value["NumericExpressionR"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["iri"]["PrefixedName"] ~= nil
		 and string.lower(value["NumericExpressionR"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["iri"]["PrefixedName"]) == "xsd:datetime"
		 ))
		 and value["NumericExpressionL"] ~= nil 
		 and value["NumericExpressionL"]["AdditiveExpression"]~= nil
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]~=nil
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]~=nil
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]~=nil
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["RDFLiteral"]~=nil
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["RDFLiteral"][1]~=nil
		then
			value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["RDFLiteral"][1] = "xsd:dateTime(" .. value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["RDFLiteral"][1] .. ")"
		end
		
		if key == "RelationalExpression" and value["Relation"] ~=nil 
		 and value["NumericExpressionL"]~=nil 
		 and value["NumericExpressionL"]["AdditiveExpression"]~= nil
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]~=nil
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]~=nil
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]~=nil
		 and ((value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["var"] ~= nil
		 and lQuery("Onto#Attribute[name='" .. string.sub(value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["var"], 2) .. "']"):is_not_empty()
		 and lQuery("Onto#Attribute[name='" .. string.sub(value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["var"], 2) .. "']"):attr("type") == "XSD_DATE")
		 or (value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["iri"] ~= nil
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["iri"]["PrefixedName"] ~= nil
		 and string.lower(value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["iri"]["PrefixedName"]) == "xsd:date"
		 ))
		 and value["NumericExpressionR"] ~= nil 
		 and value["NumericExpressionR"]["AdditiveExpression"]~= nil
		 and value["NumericExpressionR"]["AdditiveExpression"]["MultiplicativeExpression"]~=nil
		 and value["NumericExpressionR"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]~=nil
		 and value["NumericExpressionR"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]~=nil
		 and value["NumericExpressionR"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["RDFLiteral"]~=nil
		 and value["NumericExpressionR"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["RDFLiteral"][1]~=nil
		then
			value["NumericExpressionR"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["RDFLiteral"][1] = "xsd:date(" .. value["NumericExpressionR"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["RDFLiteral"][1] .. ")"
		end
		
		if key == "RelationalExpression" and value["Relation"] ~=nil 
		 and value["NumericExpressionR"]~=nil 
		 and value["NumericExpressionR"]["AdditiveExpression"]~= nil
		 and value["NumericExpressionR"]["AdditiveExpression"]["MultiplicativeExpression"]~=nil
		 and value["NumericExpressionR"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]~=nil
		 and value["NumericExpressionR"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]~=nil
		 and ((value["NumericExpressionR"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["var"] ~= nil
		 and lQuery("Onto#Attribute[name='" .. string.sub(value["NumericExpressionR"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["var"], 2) .. "']"):is_not_empty()
		 and lQuery("Onto#Attribute[name='" .. string.sub(value["NumericExpressionR"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["var"], 2) .. "']"):attr("type") == "XSD_DATE")
		 or (value["NumericExpressionR"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["iri"] ~= nil
		 and value["NumericExpressionR"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["iri"]["PrefixedName"] ~= nil
		 and string.lower(value["NumericExpressionR"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["iri"]["PrefixedName"]) == "xsd:date"
		 ))
		 and value["NumericExpressionL"] ~= nil 
		 and value["NumericExpressionL"]["AdditiveExpression"]~= nil
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]~=nil
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]~=nil
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]~=nil
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["RDFLiteral"]~=nil
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["RDFLiteral"][1]~=nil
		then
			value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["RDFLiteral"][1] = "xsd:dateTime(" .. value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["RDFLiteral"][1] .. ")"
		end
		-----------------------------------
		if key == "Filter" 
			and value["ConditionalOrExpression"]~=nil 
			and value["ConditionalOrExpression"][1]~=nil 
			and value["ConditionalOrExpression"][2]==nil 
			and value["ConditionalOrExpression"][1]["ConditionalAndExpression"]~=nil 
			and value["ConditionalOrExpression"][1]["ConditionalAndExpression"][1]~=nil 
			and value["ConditionalOrExpression"][1]["ConditionalAndExpression"][2]==nil 
			and value["ConditionalOrExpression"][1]["ConditionalAndExpression"][1]["RelationalExpression"]~=nil 
			and value["ConditionalOrExpression"][1]["ConditionalAndExpression"][1]["RelationalExpression"]["Relation"]==nil 
			and value["ConditionalOrExpression"][1]["ConditionalAndExpression"][1]["RelationalExpression"]["NumericExpressionL"]~=nil 
			and value["ConditionalOrExpression"][1]["ConditionalAndExpression"][1]["RelationalExpression"]["NumericExpressionR"] == nil 
			and value["ConditionalOrExpression"][1]["ConditionalAndExpression"][1]["RelationalExpression"]["NumericExpressionL"]["AdditiveExpression"]~= nil 
			and (value["ConditionalOrExpression"][1]["ConditionalAndExpression"][1]["RelationalExpression"]["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpressionList"]==nil 
			or (value["ConditionalOrExpression"][1]["ConditionalAndExpression"][1]["RelationalExpression"]["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpressionList"]~=nil 
			and  value["ConditionalOrExpression"][1]["ConditionalAndExpression"][1]["RelationalExpression"]["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpressionList"][1]==nil ))
			and value["ConditionalOrExpression"][1]["ConditionalAndExpression"][1]["RelationalExpression"]["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]~=nil 
			and (value["ConditionalOrExpression"][1]["ConditionalAndExpression"][1]["RelationalExpression"]["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpressionList"]==nil
			or (value["ConditionalOrExpression"][1]["ConditionalAndExpression"][1]["RelationalExpression"]["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpressionList"]~=nil and value["ConditionalOrExpression"][1]["ConditionalAndExpression"][1]["RelationalExpression"]["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpressionList"][1]==nil))
			and value["ConditionalOrExpression"][1]["ConditionalAndExpression"][1]["RelationalExpression"]["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]~=nil 
			and value["ConditionalOrExpression"][1]["ConditionalAndExpression"][1]["RelationalExpression"]["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["Additive"]==nil
			and value["ConditionalOrExpression"][1]["ConditionalAndExpression"][1]["RelationalExpression"]["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]~=nil 
			and value["ConditionalOrExpression"][1]["ConditionalAndExpression"][1]["RelationalExpression"]["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["Relation"] == nil
			and value["ConditionalOrExpression"][1]["ConditionalAndExpression"][1]["RelationalExpression"]["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["BooleanLiteral"] ~= nil
		then 
			expressionTable["Filter"] = nil
			return expressionTable
		end
		
		if key == "Filters" and (value[1]== nil or (value[1]~=nil and value[1]["Filter"]==nil and value[2]==nil))  then
			expressionTable["Filters"] = nil
			return expressionTable
		end
		
		if key== "Filter" and value["Constraint"] == nil then
			expressionTable["Filter"] = {
				Constraint = {
					BrackettedExpression = value
				}
			}
		end	
	end
	return expressionTable
end

function getSPARQLrecurtuion3(expressionTable)
	for key, value in pairs(expressionTable) do
		if type(value) == "table" then
			getSPARQLrecurtuion3(value)
		end

		if key == "RelationalExpression" and value["NumericExpressionL"]~=nil and value["NumericExpressionR"] == nil 
		 and value["NumericExpressionL"]["AdditiveExpression"]~= nil and (value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpressionList"]==nil
		 or (value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpressionList"]~=nil and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpressionList"][1]==nil))
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]~=nil and (value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpressionList"]==nil or(value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpressionList"]~=nil and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpressionList"][1]==nil))
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]~=nil and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["Additive"]==nil
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]~=nil and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["Relation"] == nil
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["GroupOrUnionGraphPattern"]~=nil 
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["GroupOrUnionGraphPattern"][1]~=nil 
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["GroupOrUnionGraphPattern"][2]== nil
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["GroupOrUnionGraphPattern"][1]["GpoupGraphPattern"]~= nil 
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["GroupOrUnionGraphPattern"][1]["GpoupGraphPattern"]["Filters"]~=nil
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["GroupOrUnionGraphPattern"][1]["GpoupGraphPattern"]["Filters"][1]~=nil 
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["GroupOrUnionGraphPattern"][1]["GpoupGraphPattern"]["Filters"][2]==nil 
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["GroupOrUnionGraphPattern"][1]["GpoupGraphPattern"]["Filters"][1]["Filter"]~=nil 
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["GroupOrUnionGraphPattern"][1]["GpoupGraphPattern"]["Filters"][1]["Filter"]["ConditionalOrExpression"]~=nil 
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["GroupOrUnionGraphPattern"][1]["GpoupGraphPattern"]["Filters"][1]["Filter"]["ConditionalOrExpression"][1]~=nil 
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["GroupOrUnionGraphPattern"][1]["GpoupGraphPattern"]["Filters"][1]["Filter"]["ConditionalOrExpression"][2]==nil 
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["GroupOrUnionGraphPattern"][1]["GpoupGraphPattern"]["Filters"][1]["Filter"]["ConditionalOrExpression"][1]["ConditionalAndExpression"]~=nil 
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["GroupOrUnionGraphPattern"][1]["GpoupGraphPattern"]["Filters"][1]["Filter"]["ConditionalOrExpression"][1]["ConditionalAndExpression"][1]~=nil 
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["GroupOrUnionGraphPattern"][1]["GpoupGraphPattern"]["Filters"][1]["Filter"]["ConditionalOrExpression"][1]["ConditionalAndExpression"][2]==nil 
		 and value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["GroupOrUnionGraphPattern"][1]["GpoupGraphPattern"]["Filters"][1]["Filter"]["ConditionalOrExpression"][1]["ConditionalAndExpression"][1]["RelationalExpression"]~=nil 
		then
			expressionTable["RelationalExpression"] = value["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["GroupOrUnionGraphPattern"][1]["GpoupGraphPattern"]["Filters"][1]["Filter"]["ConditionalOrExpression"][1]["ConditionalAndExpression"][1]["RelationalExpression"] 
		end

		if key == "NotExistsFunc" then
			local t = {
				Constraint = {
				  BuiltInCall = {
					NotExistsFunc = expressionTable["NotExistsFunc"]["Expression"]
				  }
				}
		    }
		  expressionTable["Filter"] = t
		  expressionTable["NotExistsFunc"] = nil
		  return expressionTable
		end
		
		if key == "ExistsFunc" then
			expressionTable["GroupOrUnionGraphPattern"] = value["Expression"]["GroupOrUnionGraphPattern"]
			expressionTable["ExistsFunc"]=nil
			return expressionTable
		end
		
		if key == "GpoupGraphPattern" then
			--katram izveidot triple block

			local tripleBlockTable = createTripleBlockTable(value, {})
			value["TriplesBlock"] = generateTripleBlock(tripleBlockTable)
		end
		
		if key == "GroupOrUnionGraphPattern" and value[1]~=nil and value[2]== nil and value[1]["GpoupGraphPattern"]~= nil and value[1]["GpoupGraphPattern"]["Filters"]~=nil
			and value[1]["GpoupGraphPattern"]["Filters"][1]~=nil and value[1]["GpoupGraphPattern"]["Filters"][2]==nil and value[1]["GpoupGraphPattern"]["Filters"][1]["GroupOrUnionGraphPattern"]~=nil 
			then
				expressionTable["GroupOrUnionGraphPattern"] = value[1]["GpoupGraphPattern"]["Filters"][1]["GroupOrUnionGraphPattern"]	
		end
	end
	return expressionTable
end

function createTripleBlockTable(expressionTable, t)
	for key, value in pairs(expressionTable) do
		if key == "GpoupGraphPattern" then
			return nil
		end
		if type(value) == "table" then
			createTripleBlockTable(value, t)
		end
		if key == "PrimaryExpression" then 
			local cn = className
			local varValue
			local prefixedNameValue = ""
			if value["var"]~= nil then 
				varValue = value["var"]
				if lQuery("Onto#Attribute[name='" .. string.sub(varValue, 2) .. "']"):is_not_empty() then
					prefixedNameValue = generate_SPARQL.getURI("Onto#Attribute", string.sub(varValue, 2))
					-- prefixedNameValue = "<" .. lQuery("Onto#Attribute[name='" .. string.sub(varValue, 2) .. "']"):attr("URI") .. ">"
				elseif lQuery("Onto#AllAssociation[name='" .. string.sub(varValue, 2) .. "']"):is_not_empty() then
					prefixedNameValue = generate_SPARQL.getURI("Onto#AllAssociation", string.sub(varValue, 2))
					-- prefixedNameValue = "<" .. lQuery("Onto#Attribute[name='" .. string.sub(varValue, 2) .. "']"):attr("URI") .. ">"
				else
					return
				end
			elseif value[1]~=nil and value[1]["var"]~= nil then
				varValue = value[1]["var"]
				if value["Path"]~=nil then
					for k,v in pairs(value["Path"]) do
						prefixedNameValue = prefixedNameValue .. ":" .. v .. "/"
					end
					prefixedNameValue = prefixedNameValue .. ":" .. string.sub(varValue, 2)
				elseif value["Reference"]~=nil then
					cn = value["Reference"]
					prefixedNameValue = generate_SPARQL.getURI("Onto#Attribute", string.sub(varValue, 2))
					-- prefixedNameValue = "<" .. lQuery("Onto#Attribute[name='" .. string.sub(varValue, 2) .. "']"):attr("URI") .. ">"
					varValue = value[1]["var"] .. "_" .. value["Reference"]
				else
					prefixedNameValue = generate_SPARQL.getURI("Onto#Attribute", string.sub(varValue, 2))
					-- prefixedNameValue = "<" .. lQuery("Onto#Attribute[name='" .. string.sub(varValue, 2) .. "']"):attr("URI") .. ">"
				end
			elseif value["iri"]~= nil and value["iri"]["PrefixedName"]~=nil and (value["ArgList"] == "" or value["ArgList"] == nil) then
				prefixedNameValue = value["iri"]["PrefixedName"]
				varValue = "?" .. string.sub(prefixedNameValue, string.find(prefixedNameValue, ":")+1)
			else
				return
			end
			local object = attributeAlias
			if object == nil then 
				object = varValue
			end
			t[varValue] = {
				  varOrTerm = {
					var = "?" .. cn
				  },
				  propertyListNotEmpty = {
					verb = {
					  varOrIRIref = {
						iriRef = {
						  prefixedName = prefixedNameValue
						}
					  }
					},
					objectList = {
					  object = {
						graphNode = {
						  varOrTerm = {
							var = object
						  }
						}
					  }
					}
				  }
				}
		end
	end
	return t
end

function getSPARQLrecurtuion2(expressionTable)
	for key, value in pairs(expressionTable) do
		if type(value) == "table" then
				getSPARQLrecurtuion2(value)
		end
		
		if key == "BrackettedExpression" then
			if value["NotExistsFunc"]~=nil then
				expressionTable["NotExistsFunc"] = value["NotExistsFunc"]
				expressionTable["BrackettedExpression"] = nil
				return expressionTable
			end
		end
		
		if key == "BrackettedExpression" then
			if value["ExistsFunc"]~=nil then
				expressionTable["ExistsFunc"] = value["ExistsFunc"]
				expressionTable["BrackettedExpression"] = nil
				return expressionTable
			end
		end
		
		if key == "Filter" then
			local ggpCount = 0
			local filterCount = 0
			local cORcount = 0
			local cANDcount = 0
			local t = nil
			if value["ConditionalOrExpression"]~=nil then
				for k4, v4 in pairs(value["ConditionalOrExpression"]) do
					cORcount = cORcount + 1
					for k5, v5 in pairs(v4["ConditionalAndExpression"]) do 
						cANDcount = cANDcount + 1
						if v5["RelationalExpression"]~=nil and v5["RelationalExpression"]["NumericExpressionL"]~=nil
							and v5["RelationalExpression"]["NumericExpressionL"]["AdditiveExpression"]~=nil
							and v5["RelationalExpression"]["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]~=nil
							and v5["RelationalExpression"]["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]~=nil
							and v5["RelationalExpression"]["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]~=nil
							and v5["RelationalExpression"]["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["NotExistsFunc"]~=nil
							then
								t = v5["RelationalExpression"]["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["NotExistsFunc"]
						end
					end
				end
			end
			if cORcount<2 and cANDcount<2 and t~=nil  then
				expressionTable["NotExistsFunc"] = t
				expressionTable["Filter"] = nil
				return
			end
		end
		
		if key == "Filter" then
			local ggpCount = 0
			local filterCount = 0
			local cORcount = 0
			local cANDcount = 0
			local t = nil
			if value["ConditionalOrExpression"]~=nil then
				for k4, v4 in pairs(value["ConditionalOrExpression"]) do
					cORcount = cORcount + 1
					for k5, v5 in pairs(v4["ConditionalAndExpression"]) do 
						cANDcount = cANDcount + 1
						if v5["RelationalExpression"]~=nil and v5["RelationalExpression"]["NumericExpressionL"]~=nil
							and v5["RelationalExpression"]["NumericExpressionL"]["AdditiveExpression"]~=nil
							and v5["RelationalExpression"]["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]~=nil
							and v5["RelationalExpression"]["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]~=nil
							and v5["RelationalExpression"]["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]~=nil
							and v5["RelationalExpression"]["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["ExistsFunc"]~=nil
							then
								t = v5["RelationalExpression"]["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["ExistsFunc"]
						end
					end
				end
			end
			if cORcount<2 and cANDcount<2 and t~=nil  then
				expressionTable["ExistsFunc"] = t
				expressionTable["Filter"] = nil
				return
			end
		end
	end
	return expressionTable
end

function getSPARQLrecurtuion(expressionTable)
	for key, value in pairs(expressionTable) do
		if type(value) == "table" and key~="Aggregate" and key~= "FunctionExpression" and key~= "ArgListExpression" and key~= "RegexExpression"
		and key~= "SubstringExpression" and key ~= "SubstringBifExpression" and key ~= "StrReplaceExpression" and key~="ExpressionList"	then
			getSPARQLrecurtuion(value)
		end
		
		if (key == "Aggregate" or key == "FunctionExpression" or key == "ArgListExpression" or key == "RegexExpression" or key=="SubstringExpression" or key=="SubstringBifExpression" or key == "StrReplaceExpression" or key == "ExpressionList") 
		and value["Expression"]~=nil and value["Expression"]["OrExpression"]~=nil and value["Expression"]["OrExpression"][1]~=nil
		 and value["Expression"]["OrExpression"][1]["ANDExpression"]~=nil
		 and value["Expression"]["OrExpression"][1]["ANDExpression"][1]~=nil
		 and value["Expression"]["OrExpression"][1]["ANDExpression"][1]["ConditionalOrExpression"]~=nil
		then
			value["Expression"]["ConditionalOrExpression"] = value["Expression"]["OrExpression"][1]["ANDExpression"][1]["ConditionalOrExpression"]
			value["Expression"]["OrExpression"] = nil
		end
		
		if key == "ExpressionList" then
			table.insert(expressionTable, expressionTable["ExpressionList"])
		end
		
		if (key == "PrimaryExpression" or (key == "UnaryExpression" and value["Additive"]==nil)
		 or (key == "MultiplicativeExpression" and (value["UnaryExpressionList"]==nil or (value["UnaryExpressionList"]~=nil and value["UnaryExpressionList"][1]==nil))) 
		 or (key == "AdditiveExpression" and (value["MultiplicativeExpressionList"]==nil or (value["MultiplicativeExpressionList"]~=nil and value["MultiplicativeExpressionList"][1]==nil)))
		 or (key == "NumericExpressionL")
		 or (key == "RelationalExpression" and value["Relation"]==nil)
		 ) and value["NotExistsFunc"]~=nil then
			expressionTable["NotExistsFunc"] = value["NotExistsFunc"]
			expressionTable[key] = nil
			return
		end
		
		if (key == "PrimaryExpression" or (key == "UnaryExpression" and value["Additive"]==nil)
		 or (key == "MultiplicativeExpression" and (value["UnaryExpressionList"]==nil or (value["UnaryExpressionList"]~=nil and value["UnaryExpressionList"][1]==nil))) 
		 or (key == "AdditiveExpression" and (value["MultiplicativeExpressionList"]==nil or (value["MultiplicativeExpressionList"]~=nil and value["MultiplicativeExpressionList"][1]==nil)))
		 or (key == "NumericExpressionL")
		 or (key == "RelationalExpression" and value["Relation"]==nil)
		 ) and value["ExistsFunc"]~=nil then
			expressionTable["ExistsFunc"] = value["ExistsFunc"]
			expressionTable[key] = nil
			return
		end
		
		if (key == "ConditionalAndExpression" or  key == "ConditionalOrExpression" or key == "ANDExpression" or key == "OrExpression") and value[1]~=nil and value[1]["NotExistsFunc"]~=nil then
			expressionTable["NotExistsFunc"] = value[1]["NotExistsFunc"]
			expressionTable[key] = nil
			return expressionTable
		end
		
		if (key == "ConditionalAndExpression" or  key == "ConditionalOrExpression" or key == "ANDExpression" or key == "OrExpression") and value[1]~=nil and value[1]["ExistsFunc"]~=nil then
			expressionTable["ExistsFunc"] = value[1]["ExistsFunc"]
			expressionTable[key] = nil
			return expressionTable
		end
		
		if key == "ConditionalAndExpression" then
			local filterTable, hasConstraint = getFilterCondAnd2(value)
			if #filterTable ~= 0 then 
				local filter = {ConditionalAndExpression = filterTable}
				if hasConstraint then 
					filter = filterTable[1]
				end
				expressionTable["ConditionalAndExpression"]=nil
				expressionTable["Filter"] = filter
				return expressionTable
			end
		end
		
		if key == "ConditionalOrExpression" then
			local filterTable, hasConstraint = getFilterCondOr(value)
			if #filterTable ~= 0 then 
				local filter = {ConditionalOrExpression = filterTable}
				if hasConstraint then 
					filter = filterTable[1]
				end
				expressionTable["ConditionalOrExpression"]=nil
				expressionTable["Filter"] = filter
				return expressionTable
			end
		end
		
		if key == "ANDExpression" then
			--apvienot tripleBlock
			local filterTable, triplesBlockTable = getFilterAND(value)
			expressionTable["Filters"] = filterTable
			
			if #filterTable ~= 0 or #triplesBlockTable ~= 0 or triplesBlockTable["TriplesSameSubject"]~=nil then 
				expressionTable["ANDExpression"] = nil
				return
			end
		end
		
		if key == "OrExpression" then
			expressionTable["GroupOrUnionGraphPattern"] = groupGraphPatternTable(value)
			expressionTable["OrExpression"] = nil
		end
		
		if key == "BrackettedExpression" then
			if value["GroupOrUnionGraphPattern"]~=nil then
				expressionTable["GroupOrUnionGraphPattern"] = getBElower(value["GroupOrUnionGraphPattern"])
				expressionTable["BrackettedExpression"] = nil
				return expressionTable
			end
		end
		
		if key == "PrimaryExpression" and value["GroupOrUnionGraphPattern"]~= nil then
			local ggpCount = 0
			local filterCount = 0
			for k, v in pairs(value["GroupOrUnionGraphPattern"]) do
				ggpCount = ggpCount + 1
				for k1,v1 in pairs(v["GpoupGraphPattern"]) do
					for k2, v2 in pairs(v1) do
						filterCount = filterCount + 1
					end
				end
			end
			if ggpCount<2 and filterCount<2 then
				local t = value["GroupOrUnionGraphPattern"][1]["GpoupGraphPattern"]["Filters"][1]["Filter"]["BrackettedExpression"]
				value["BrackettedExpression"] = t
				value["GroupOrUnionGraphPattern"] = nil
				return
			end
		end
		
		if key == "RelationalExpression" then
			if value[1]~= nil and value[1]["GroupOrUnionGraphPattern"]~=nil then
				for k,v in pairs(value) do
					if v["GroupOrUnionGraphPattern"]~= nil then 
						v["ConditionalOrExpression"] = v["GroupOrUnionGraphPattern"][1]["GpoupGraphPattern"]["Filters"][1]["Filter"]["ConditionalOrExpression"]
						v["GroupOrUnionGraphPattern"]=nil
					end
				end
			end
		end
		
		if key == "RelationalExpression" and (value["Relation"] == "IN" or value["Relation"] == "NOT IN") then
			for k,v in pairs(value) do
				if v["ConditionalOrExpression"]~=nil then
					if v["ConditionalOrExpression"][1]["ConditionalAndExpression"][1]["RelationalExpression"]["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["var"]~= nil then
						v["ConditionalOrExpression"][1]["ConditionalAndExpression"][1]["RelationalExpression"]["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["RDFLiteral"] = {'"' .. string.sub(v["ConditionalOrExpression"][1]["ConditionalAndExpression"][1]["RelationalExpression"]["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["var"], 2) .. '"'}
						v["ConditionalOrExpression"][1]["ConditionalAndExpression"][1]["RelationalExpression"]["NumericExpressionL"]["AdditiveExpression"]["MultiplicativeExpression"]["UnaryExpression"]["PrimaryExpression"]["var"] = nil
					end
				end
			end
		end
	end
	
	return expressionTable
end

function getBElower(valueTable)
	for key, value in pairs(valueTable) do
		if key == "Filters" then
			for k, v in pairs(value) do
				local t = v["Filter"]
				v["Filter"] = {BrackettedExpression = t}
				return
			end
		end
		if type(value) == "table" then
			getBElower(value)
		end
	end
	return valueTable
end

function groupGraphPatternTable(valueTable)
	local t ={}
	for key, value in pairs(valueTable) do
		table.insert(t, {GpoupGraphPattern = value})
	end
	return t
end

function getFilterAND(valueTable)
	local filtersTable = {}
	local triplesBlockTable = {}
	for key, value in pairs(valueTable) do
		if (value["Filter"]~=nil) then 
			table.insert(filtersTable, {Filter = value["Filter"]})
		end
		if (value["TriplesBlock"]~=nil) then 
			if triplesBlockTable["TriplesSameSubject"] == nil then
				triplesBlockTable = value["TriplesBlock"]
			else
				if triplesBlockTable["TriplesBlock"]==nil then
					triplesBlockTable["TriplesBlock"] = value["TriplesBlock"]
				else
					local t = value["TriplesBlock"]
					t["TriplesBlock"] = triplesBlockTable
					triplesBlockTable = t
				end
			end
			-- table.insert(filtersTable, {TriplesBlock = triplesBlockTable})
		end
	end
	return filtersTable, triplesBlockTable
end

function getFilterCondOr(condValueTable)
	local filtersTable = {}
	local hasConstraint = false
	for key, value in pairs(condValueTable) do
		if (value["Filter"]~=nil and value["Filter"]["Constraint"] ~= nil) then 
			hasConstraint = true
		end
		if (value["Filter"]~=nil) then 
			table.insert(filtersTable, value["Filter"])
		end
	end
	return filtersTable, hasConstraint
end

function getFilterCondAnd2(condValueTable)
	local filtersTable = {}
	local hasConstraint = false
	for key, value in pairs(condValueTable) do
		if (value["RelationalExpression"]~=nil) then 
			table.insert(filtersTable, {RelationalExpression = value["RelationalExpression"]})
		end
	end
	return filtersTable, hasConstraint
end

function parsesparqlExpressionGrammar(str, name, alias, attributesNamesTablePrevious, idT, classInProcessing)
	classInProc = classInProcessing
	idTable = idT
	attributesNamesTable = {}
	attributesNamesTableP = attributesNamesTablePrevious
	prefixes = {}
	className = name
	attributeAlias = alias
	
	local gramarxsd = [[
		main <- (({[^A-Za-z0-9_]*} ({"xsd:" string/string}))* ({[^A-Za-z0-9_]*})?) -> {}
		string <- ([A-Za-z0-9_] [A-Za-z0-9_]*)
	]]
	local parsedTablexsd = re.match(str, gramarxsd)
	local replacementTablexsd = replacementTablexsd()
	local newString2 = ""
	for _, a in pairs(parsedTablexsd) do 
		if replacementTablexsd[string.lower(a)]~= nil then newString2 = newString2 .. replacementTablexsd[string.lower(a)]
		else newString2 = newString2 .. a end
	end
	
	local gramar = [[
		main <- (({[^A-Za-z0-9_]*} string)* ({[^A-Za-z0-9_]*})?) -> {}
		string <- ({[A-Za-z0-9_] [A-Za-z0-9_]*})
	]]
	
	local newString = ""
	local parsedTable = re.match(newString2, gramar)
	local replacementTable = replacementTable()
	for _, a in pairs(parsedTable) do 
		if replacementTable[string.lower(a)]~= nil then  newString = newString .. replacementTable[string.lower(a)]
		else newString = newString .. a end
	end
	
	local gramar = [[
		main <- (({[^A-Za-z0-9_]*} ("xsd:" string/string))* ({[^A-Za-z0-9_]*})?) -> {}
		string <- ({[A-Za-z0-9_] [A-Za-z0-9_]*})
	]]
	local parsedTable = re.match(str, gramar)
	-- print("GGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGG",dumptable(parsedTable))
	-- print(newString)
	local expressionTable = re.match(newString, sparqlExpressionGrammar())
	-- print(dumptable(expressionTable) , re.match(newString, sparqlExpressionGrammar()) or "NOT")
	
	if expressionTable == nil then
		incorrectSyntaxForm(newString)
		expressionTable = {}
	end
	-- print(dumptable(expressionTable))
	local ggpTable = getSPARQLrecurtuion(expressionTable)
	local ggpTable2 = getSPARQLrecurtuion2(ggpTable)
	local ggpTable3 = getSPARQLrecurtuion3(ggpTable2)
	local ggpTable4 = getSPARQLrecurtuion4(ggpTable3)
	-- print(dumptable(ggpTable))
	-- print(dumptable(attributesNamesTable))
	-- print(getSPARQLGpoupGraphPatternExpression(ggpTable4, ""))
	return getSPARQLGpoupGraphPatternExpression(ggpTable4, ""), attributesNamesTable, prefixes
end

function incorrectSyntaxForm(expression)

  local close_button = lQuery.create("D#Button", {
    caption = "Close"
    ,eventHandler = utilities.d_handler("Click", "lua_engine", "lua.sparql_grammar.close()")
  })
  
  local form = lQuery.create("D#Form", {
    id = "incorrectSyntax"
    ,caption = "Incorrect Syntax"
    ,buttonClickOnClose = false
    ,cancelButton = close_button
    ,defaultButton = close_button
    ,eventHandler = utilities.d_handler("Close", "lua_engine", "lua.sparql_grammar.close()")
    ,minimumWidth = 410
	,component = {
		lQuery.create("D#HorizontalBox",{
			component = {
				lQuery.create("D#VerticalBox",{
					component = {
						lQuery.create("D#Label", {caption = "Incorrect syntax in"})
						,lQuery.create("D#Label", {caption = expression})
					}
				})
			}})
		,lQuery.create("D#HorizontalBox", {
			horizontalAlignment = 1
			,component = {close_button}
		})
    }
  })
  dialog_utilities.show_form(form)

end

function close()
	lQuery("D#Event"):delete()
	utilities.close_form("incorrectSyntax")
end

function parsesparqlExpressionGrammar2(str, name, alias, attrNamesTable, attributesNamesTablePrevious, idT, classInProcessing)
	idTable = idT
	attributesNamesTableP = attributesNamesTablePrevious
	className = name
	attributeAlias = alias
	classInProc = classInProcessing
	prefixes = {}
	pathInString = false
	
	local gramarxsd = [[
		main <- (({[^A-Za-z0-9_]*} ({"xsd:" string/string}))* ({[^A-Za-z0-9_]*})?) -> {}
		string <- ([A-Za-z0-9_] [A-Za-z0-9_]*)
	]]
	local parsedTablexsd = re.match(str, gramarxsd)
	local replacementTablexsd = replacementTablexsd()
	local newString2 = ""
	for _, a in pairs(parsedTablexsd) do 
		if replacementTablexsd[string.lower(a)]~= nil then newString2 = newString2 .. replacementTablexsd[string.lower(a)]
		else newString2 = newString2 .. a end
	end
	
	
	local gramar = [[
		main <- (({[^A-Za-z0-9_]*} string)* ({[^A-Za-z0-9_]*})?) -> {}
		string <- ({[A-Za-z0-9_] [A-Za-z0-9_]*})
	]]
	
	local newString = ""
	local parsedTable = re.match(newString2, gramar)
	local replacementTable = replacementTable()
	for _, a in pairs(parsedTable) do 
		if replacementTable[string.lower(a)]~= nil then newString = newString .. replacementTable[string.lower(a)]
		else newString = newString .. a end
	end
	
	
	
	local expressionTable = re.match(newString, sparqlExpressionGrammar())
	local et = expressionTable
	local ggpet2 =  getSPARQLGpoupGraphPatternExpression(et, "") 
	local triplesString = ""
	if string.find(ggpet2, "%(") or string.find(ggpet2, ",") or string.find(ggpet2, " ") or pathInString == true then
		local tripleBlockTable = createTripleBlockTable(expressionTable, {})	
		local triple = generateTripleBlock(tripleBlockTable)
		if triple~= nil then triplesString = getSPARQLGpoupGraphPatternExpression(triple, "") end 
		if expressionTable == nil then
			incorrectSyntaxForm(newString)
			expressionTable = {}
		end
	end
	isAggregate = false
	isFunction  = false
	attributesNamesTable = attrNamesTable
	local ggpet =  getSPARQLGpoupGraphPatternExpression(expressionTable, "") 
	return ggpet, isAggregate, triplesSameSubjectTable, classTriplesTable, isFunction, attributesNamesTable, triplesString, prefixes
end

function generateTripleBlock(triplesSameSubjectTable)
	if next (triplesSameSubjectTable) ~= nil then
		local t = {}
		for key, value in pairs(triplesSameSubjectTable) do
			t["TriplesSameSubject"] = value
			triplesSameSubjectTable[key] = nil
			t["TriplesBlock"] = generateTripleBlock(triplesSameSubjectTable)
		end
		return t
	end
	return
end

function replacementTablexsd()
	local replacementTablexsd = {
		["date"] = "xsd:date",
		["datetime"] = "xsd:dateTime",
		["string"] = "xsd:string",
		["integer"] = "xsd:integer",
		["boolean"] = "xsd:boolean",
		["gYear"] = "xsd:gYear",
		["gMonth"] = "xsd:gMonth",
		["gDay"] = "xsd:gDay",
	}
	return replacementTablexsd
end

function replacementTable()
	local replacementTable = {
		["in"] = "IN",
		["not"] = "NOT",
		["str"] = "STR",
		["lang"] = "LANG",
		["langmatches"] = "LANGMATCHES",
		["datatype"] = "DATATYPE",
		["bound"] = "BOUND",
		["iri"] = "IRI",
		["uri"] = "URI",
		["bnode"] = "BNODE",
		["rand"] = "RAND",
		["abs"] = "ABS",
		["ceil"] = "CEIL",
		["floor"] = "FLOOR",
		["round"] = "ROUND",
		["concat"] = "CONCAT",
		["strlen"] = "STRLEN",
		["ucase"] = "UCASE",
		["lcase"] = "LCASE",
		["encode_for_uri"] = "ENCODE_FOR_URI",
		["contains"] = "CONTAINS",
		["strstarts"] = "STRSTARTS",
		["strends"] = "STRENDS",
		["strbefore"] = "STRBEFORE",
		["strafter"] = "STRAFTER",
		["year"] = "YEAR",
		["month"] = "MONTH",
		["day"] = "DAY",
		["hours"] = "HOURS",
		["minutes"] = "MINUTES",
		["seconds"] = "SECONDS",
		["timezone"] = "TIMEZONE",
		["tz"] = "TZ",
		["now"] = "NOW",
		["uuid"] = "UUID",
		["struuid"] = "STRUUID",
		["md5"] = "MD5",
		["sha1"] = "SHA1",
		["sha256"] = "SHA256",
		["sha384"] = "SHA384",
		["sha512"] = "SHA512",
		["coalesce"] = "COALESCE",
		["if"] = "IF",
		["strlang"] = "STRLANG",
		["strdt"] = "STRDT",
		["sameterm"] = "sameTerm",
		["isiri"] = "isIRI",
		["isuri"] = "isURI",
		["isblank"] = "isBLANK",
		["isliteral"] = "isLITERAL",
		["isnumeric"] = "isNUMERIC",
		["distinct"] = "DISTINCT",
		["true"] = "true",
		["false"] = "true",
		["regex"] = "REGEX",
		["substr"] = "SUBSTR",
		["replace"] = "REPLACE",
		["exists"] = "EXISTS",
		["count"] = "COUNT",
		["sum"] = "SUM",
		["min"] = "MIN",
		["max"] = "MAX",
		["avg"] = "AVG",
		["sample"] = "SAMPLE",
		["group_concat"] = "GROUP_CONCAT",
		["separator"] = "SEPARATOR",
		["or"] = "OR",
		["and"] = "AND",
		["substring"] = "SUBSTRING",
	}
	return replacementTable
end